@@ -366,4 +366,58 @@ def toy_significance(**kwargs):
     
     
     
-    
+@click.command(name='toy_limit')
+@click.option('-i', '--input_file', 'filename', required=True, 
+              help='Path to the input workspace file.')
+@click.option('-d', '--data', 'data_name', default='combData', show_default=True,
+              help='Name of the dataset used for computing observed limit.')
+@click.option('-o', '--output_file', 'outname', 
+              default="toy_study/toy_result_seed_{seed}_batch_{batch}.root",
+              show_default=True,
+              help='Name of the output file containing toy results.')
+@click.option('--poi_max', type=float, default=None,
+              help='Maximum range of POI.')
+@click.option('--poi_min', type=float, default=None,
+              help='Minimum range of POI.')
+@click.option('--scan_max', type=float, default=None,
+              help='Maximum scan value of POI.')
+@click.option('--scan_min', type=float, default=None,
+              help='Minimum scan value of POI.')
+@click.option('--steps', type=int, default=10, show_default=True,
+              help='Number of scan steps.')
+@click.option('--mu_val', type=float, default=None,
+              help='Value of POI for running a single point')
+@click.option('-n', '--n_toys', type=int,
+              help='Number of the toys to use.')
+@click.option('-b', '--batchsize', type=int, default=50, show_default=True,
+              help='Divide the task into batches each containing this number of toys. '
+                   'Result from each batch is saved for caching and different batches '
+                   'are run in parallel if needed')
+@click.option('-s', '--seed', type=int, default=2021,  show_default=True,
+              help='Random seed used for generating toy datasets.')
+@click.option('-t', '--tolerance', type=float, default=1.,  show_default=True,
+              help='Tolerance for minimization.')
+@click.option('-p', '--poi', 'poi_name', default=None,
+              help='Name of the parameter of interest (POI). If None, the first POI is used.')
+@click.option('--minimizer_type', default="Minuit2", show_default=True,
+              help='Minimizer type')
+@click.option('--strategy', type=int, default=1, show_default=True,
+              help='Default minimization strategy')
+@click.option('--print_level', type=int, default=-1, show_default=True,
+              help='Minimizer print level')
+@click.option('--verbosity', default="INFO", show_default=True,
+              help='Quickstats internal verbosity level.')
+@click.option('--snapshot', 'snapshot_name', default=None, help='Name of initial snapshot')
+@click.option('--parallel', type=int, default=-1, show_default=True,
+              help='Parallelize job across different scan values.'+\
+                   'Use -1 for N_CPU workers.')
+def toy_limit(**kwargs):
+    """
+    Generate toys and evaluate limits
+    """
+    from quickstats.components.toy_limit_calculator import evaluate_batched_toy_limits
+    if not (((kwargs['scan_min'] is None) and (kwargs['scan_max'] is None) and (kwargs['mu_val'] is not None)) or \
+           ((kwargs['scan_min'] is not None) and (kwargs['scan_max'] is not None) and (kwargs['mu_val'] is None))):
+        raise ValueError("please provide either (scan_min, scan_max, steps) for running a scan or (mu_val)"
+                         " for running a single point")        
+    evaluate_batched_toy_limits(**kwargs)
\ No newline at end of file
