@@ -60,4 +60,11 @@ def get_ws_datasets(input_file, ws_name, mc_name, output_file=None, patterns=Non
 @click.option('-o', '--output_file', default=None, help='Export output to text file')
 @click.option('-p', '--patterns', default=None, help='Match POIs with given patterns (separated by commas).')
 def get_ws_pois(input_file, ws_name, mc_name, output_file=None, patterns=None):
-    get_ws_attributes('pois', input_file, ws_name, mc_name, output_file, patterns)    
\ No newline at end of file
+    get_ws_attributes('pois', input_file, ws_name, mc_name, output_file, patterns)
+    
+@inspect_ws.command(name='summary')
+@click.option('-i', '--input_file', required=True, help='Path to the input workspace file')
+@click.option('-o', '--output_file', default=None, help='Export output to text file. If None, no output is saved.')
+def get_summary(input_file, output_file=None):
+    model = ExtendedModel(input_file, verbosity="ERROR", data_name=None)
+    model.print_summary(save_as=output_file)
\ No newline at end of file
