@@ -5,9 +5,11 @@ import json
 
 import numpy as np
 
+from quickstats import semistaticmethod
 from quickstats.components import AnalysisObject
 from quickstats.utils.root_utils import load_macro
 from quickstats.utils import common_utils
+from quickstats.utils.io import Verbosity
 
 import ROOT
 
@@ -35,7 +37,7 @@ class AsymptoticCLs(AnalysisObject):
                  num_cpu:int=1, offset:bool=True, optimize:int=2,
                  fix_cache:bool=True, fix_multi:bool=True, max_calls:int=-1,
                  max_iters:int=-1, constrain_nuis:bool=True, batch_mode:bool=False,
-                 int_bin_precision:float=-1., verbosity:Union[int, str]="INFO", **kwargs):
+                 int_bin_precision:float=-1., verbosity:Optional[Union[int, str]]="INFO", **kwargs):
         """
         args:
             precision: precision in mu that defines iterative cutoff
@@ -98,27 +100,26 @@ class AsymptoticCLs(AnalysisObject):
         self.nom_glob_name = names['nominal_globs']
         self.nom_nuis_name = names['nominal_nuis']
         self.nll_snapshot_name = names['nll_snapshot']
-        
         self.load_extension()
         self.LimitTool = ROOT.AsymptoticCLsTool(1 - self.CL, precision, do_tilde)
 
-    @staticmethod
-    def load_extension():
+    @semistaticmethod
+    def load_extension(self):
         try:
             if not hasattr(ROOT, 'AsymptoticCLsTool'):
                 result = load_macro('AsymptoticCLsTool')
                 if hasattr(ROOT, 'AsymptoticCLsTool'):
-                    print('INFO: Loaded extension module "AsymptoticCLsTool"')
+                    self.stdout.info('INFO: Loaded extension module "AsymptoticCLsTool"')
         except Exception as e:
             print(e)
             
     def set_poi(self, poi_name:Optional[str]=None):
         if poi_name is None:
             self.poi = self.model.pois.first()
-            print('INFO: POI name not given. Set to first poi "{}" by default.'.format(self.poi.GetName()))
+            self.stdout.info('INFO: POI name not given. Set to first poi "{}" by default.'.format(self.poi.GetName()))
         else:
             self.poi = self.model.get_poi(poi_name)
-            print('INFO: POI set to "{}"'.format(poi_name))
+            self.stdout.info('INFO: POI set to "{}"'.format(poi_name))
         
     def set_poi_value(self, val:float):
         if math.isnan(val):
@@ -139,7 +140,7 @@ class AsymptoticCLs(AnalysisObject):
         snapshot = self.ws.getSnapshot(name)
         if not snapshot:
             raise RuntimeError('snapshot "{}" does not exist'.format(name))
-        #print('INFO: Loaded snapshot "{}"'.format(name))
+        self.stdout.debug('INFO: Loaded snapshot "{}"'.format(name))
         self.ws.loadSnapshot(name)
             
     def save_nll_snapshot(self, nll, mu):
@@ -186,6 +187,8 @@ class AsymptoticCLs(AnalysisObject):
         self.poi.setConstant(1)
         self.set_poi_value(mu)
         nll_value = self.get_nll_val(nll)
+        if Verbosity.DEBUG >= self.stdout.verbosity:
+            self.model.print_summary(items=['nuisance_parameter', 'global_observable'])        
         self.poi.setConstant(is_const)
         qmu = 2*(nll_value - nll_muhat)
         if qmu < 0 :
@@ -213,8 +216,8 @@ class AsymptoticCLs(AnalysisObject):
     def get_limit(self, nll:"ROOT.RooNLLVar", initial_guess:float, summary_label:Optional[int]=None):
         self.global_status = 0
         nll_name = nll.GetName()
-        print("----------------------------------")
-        print("Getting limit for nll: ", nll_name)
+        self.stdout.info("----------------------------------")
+        self.stdout.info(f"Getting limit for nll: {nll_name}")
         self.poi.setConstant(0)
         asimov_0_nll = self.summary[0]['nll']
         if nll == asimov_0_nll:
@@ -247,15 +250,15 @@ class AsymptoticCLs(AnalysisObject):
         CLs = self.LimitTool.calcCLs(qmu, sigma_b, mu_guess)
         qmu95 = self.LimitTool.getQmu95(sigma_b, mu_guess)
         self.set_poi_value(mu_guess)
-        print("Initial guess:  ", mu_guess)
-        print("Sigma(obs):     ", sigma_guess)
-        print("Sigma(mu,0):    ", sigma_b)
-        print("muhat:          ", mu_hat)
-        print("qmu95:          ", qmu95)
-        print("qmu:            ", qmu)
-        print("pmu:            ", pmu)
-        print("1-pb:           ", pb)
-        print("CLs:            ", CLs)
+        self.stdout.info(f"Initial guess:  {mu_guess}")
+        self.stdout.info(f"Sigma(obs):     {sigma_guess}")
+        self.stdout.info(f"Sigma(mu,0):    {sigma_b}")
+        self.stdout.info(f"muhat:          {mu_hat}")
+        self.stdout.info(f"qmu95:          {qmu95}")
+        self.stdout.info(f"qmu:            {qmu}")
+        self.stdout.info(f"pmu:            {pmu}")
+        self.stdout.info(f"1-pb:           {pb}")
+        self.stdout.info(f"CLs:            {CLs}")
 
         n_damping = 1
         guess_to_corr = {}
@@ -265,8 +268,8 @@ class AsymptoticCLs(AnalysisObject):
         mu_pre2 = mu_hat
         
         while (abs(mu_pre-mu_guess) > self.precision*mu_guess):
-            print('----------------------------------')
-            print('Starting iteration {} of {}'.format(n_iter, nll_name))
+            self.stdout.info('----------------------------------')
+            self.stdout.info('Starting iteration {} of {}'.format(n_iter, nll_name))
             # do this to avoid comparing multiple minima in the conditional and unconditional fits           
             if (n_iter == 0):
                 self.load_nll_snapshot(nll, mu_hat)
@@ -299,7 +302,7 @@ class AsymptoticCLs(AnalysisObject):
                 if (abs(first - (mu_guess - corr)) < self.direction*mu_guess*0.02) and \
                 (abs(corr) > self.direction*mu_guess*self.precision):
                     damping_factor *= 0.8
-                    print('Changing damping factor to {}, n_damping = {}'.format(damping_factor, n_damping))
+                    self.stdout.info('Changing damping factor to {}, n_damping = {}'.format(damping_factor, n_damping))
                     n_damping += 1
                     if n_damping > 10:
                         n_damping = 1
@@ -317,20 +320,20 @@ class AsymptoticCLs(AnalysisObject):
             CLs = self.LimitTool.calcCLs(qmu, sigma_b, mu_pre)
             qmu95 = self.LimitTool.getQmu95(sigma_b, mu_pre)
             
-            print("NLL:            ", nll_name)
-            print("Previous guess: ", mu_pre)
-            print("Sigma(obs):     ", sigma_guess)
-            print("Sigma(mu,0):    ", sigma_b)
-            print("muhat:          ", mu_hat)
-            print("pmu:            ", pmu)
-            print("1-pb:           ", pb)
-            print("CLs:            ", CLs)
-            print("qmu95:          ", qmu95)
-            print("qmu:            ", qmu)
-            print("qmuA0:          ", qmuA)
-            print("Precision:      ", self.direction*mu_guess*self.precision)
-            print("Correction:     ", -corr)
-            print("New guess:      ", mu_guess)
+            self.stdout.info(f"NLL:            {nll_name}")
+            self.stdout.info(f"Previous guess: {mu_pre}")
+            self.stdout.info(f"Sigma(obs):     {sigma_guess}")
+            self.stdout.info(f"Sigma(mu,0):    {sigma_b}")
+            self.stdout.info(f"muhat:          {mu_hat}")
+            self.stdout.info(f"pmu:            {pmu}")
+            self.stdout.info(f"1-pb:           {pb}")
+            self.stdout.info(f"CLs:            {CLs}")
+            self.stdout.info(f"qmu95:          {qmu95}")
+            self.stdout.info(f"qmu:            {qmu}")
+            self.stdout.info(f"qmuA0:          {qmuA}")
+            self.stdout.info(f"Precision:      {self.direction*mu_guess*self.precision}")
+            self.stdout.info(f"Correction:     {-corr}")
+            self.stdout.info(f"New guess:      {mu_guess}")
             
             n_iter += 1
             if (n_iter > 25):
@@ -357,8 +360,8 @@ class AsymptoticCLs(AnalysisObject):
                 self.summary[summary_label] = summary
             else:
                 self.summary[summary_label].update(summary)
-        print('Found limit for nll {}: {}'.format(nll_name, mu_guess))
-        print('Finished in {} itreations'.format(n_iter))
+        self.stdout.info('Found limit for nll {}: {}'.format(nll_name, mu_guess))
+        self.stdout.info('Finished in {} itreations'.format(n_iter))
         return mu_guess
     
     @staticmethod
@@ -397,12 +400,12 @@ class AsymptoticCLs(AnalysisObject):
     def print_failures(self):
         has_failtures = any(self.summary[sigma].get('status', 0) > 0 for sigma in self.summary)
         if has_failtures:
-            print('-----------------------------------------------')
-            print('Unresolved fit failures detected')
+            self.stdout.info('-----------------------------------------------')
+            self.stdout.info('Unresolved fit failures detected')
             for sigma in self.summary:
                 label = self._SIGMA_LABEL_.get(sigma, sigma)
-                print("{}:".format(label).ljust(10), self.summary[sigma].get('status', None))
-            print('-----------------------------------------------')
+                self.stdout.info("{}:".format(label).ljust(10), self.summary[sigma].get('status', None))
+            self.stdout.info('-----------------------------------------------')
 
     def print_summary(self):
         if (not self.limits) or (not self.approx_limits):
@@ -411,21 +414,21 @@ class AsymptoticCLs(AnalysisObject):
         print('')
         self.print_failures()
         if self.do_better_bands:
-            print('Guess for bands')
-        print('+2sigma:  ', self.approx_limits[2])
-        print('+1sigma:  ', self.approx_limits[1])
-        print('-1sigma:  ', self.approx_limits[-1])
-        print('-2sigma:  ', self.approx_limits[-2])
+            self.stdout.info('Guess for bands')
+        self.stdout.info(f'+2sigma:  {self.approx_limits[2]}')
+        self.stdout.info(f'+1sigma:  {self.approx_limits[1]}')
+        self.stdout.info(f'-1sigma:  {self.approx_limits[-1]}')
+        self.stdout.info(f'-2sigma:  {self.approx_limits[-2]}')
         if self.do_better_bands:
-            print('\nCorrect bands')
-            print('+2sigma:  ', self.limits[2])
-            print('+1sigma:  ', self.limits[1])
-            print('-1sigma:  ', self.limits[-1])
-            print('-2sigma:  ', self.limits[-2])
-        print('Injected: ', self.limits['inj'])
-        print('Median:   ', self.limits[0])
-        print('Observed: ', self.limits['obs'])
-        print('')
+            self.stdout.info('\nCorrect bands')
+            self.stdout.info(f'+2sigma:  {self.limits[2]}')
+            self.stdout.info(f'+1sigma:  {self.limits[1]}')
+            self.stdout.info(f'-1sigma:  {self.limits[-1]}')
+            self.stdout.info(f'-2sigma:  {self.limits[-2]}')
+        self.stdout.info('Injected: {}'.format(self.limits['inj']))
+        self.stdout.info(f'Median:   {self.limits[0]}')
+        self.stdout.info('Observed: {}'.format(self.limits['obs']))
+        self.stdout.info('')
         
     def save(self, filename:str='limits.json', summary=True):
         json.dump(self.limits, open(filename, 'w'), indent=2)
@@ -446,7 +449,7 @@ class AsymptoticCLs(AnalysisObject):
         n_times_sigma = self.get_limit(asimov_0_nll, mu_guess)
         status = self.global_status
         sigma = n_times_sigma/n
-        print('Found N * sigma = {} * {}'.format(n, sigma))
+        self.stdout.info('Found N * sigma = {} * {}'.format(n, sigma))
         snapshot_0_name = self.nll_maps[asimov_0_nll]['snapshot']
         self.load_snapshot(snapshot_0_name)
         asimov_data_n = self.model.generate_asimov(self.poi.GetName(), poi_val=n_times_sigma,
@@ -529,5 +532,5 @@ class AsymptoticCLs(AnalysisObject):
         
         self.print_summary()
         
-        print('INFO: Finished with {} calls to minimize(nll)'.format(self.minimizer_calls))
+        self.stdout.info('INFO: Finished with {} calls to minimize(nll)'.format(self.minimizer_calls))
         return limits
\ No newline at end of file
