@@ -9,9 +9,9 @@ import math
 import numpy as np
 import ROOT
 
-from quickstats import _PRINT_
+from quickstats.components import AbstractObject
 
-class ExtendedMinimizer(object):
+class ExtendedMinimizer(AbstractObject):
     _SCAN_EXCLUDE_CONFIG_ = ['scan', 'minos', 'save']
     _NLL_COMMANDS_ = ['NumCPU', 'Constrain', 'CloneData', 'GlobalObservables', 
                       'GlobalObservablesTag', 'OffsetLikelihood', 'Offset']
@@ -51,7 +51,8 @@ class ExtendedMinimizer(object):
         return self._config
     
     def __init__(self, minimizer_name:str, pdf:"ROOT.RooAbsPdf", data:"ROOT.RooAbsData",
-                 minimizer_options:Optional[Dict]=None):
+                 minimizer_options:Optional[Dict]=None, verbosity:Optional[Union[int, str]]="INFO"):
+        super().__init__(verbosity=verbosity)
         """
             Minimizr Options:
                 precision: precision for the find sigma algorithm
@@ -83,7 +84,7 @@ class ExtendedMinimizer(object):
         
         self.configure_default_minimizer_options()
         
-        _PRINT_.info('INFO: Created ExtendedMinimizer("{}") instance'.format(self.name))
+        self.stdout.info('INFO: Created ExtendedMinimizer("{}") instance'.format(self.name))
         
     @property
     def data(self):
@@ -228,7 +229,7 @@ class ExtendedMinimizer(object):
         perform_minimization = any(not attached.isConstant() for attached in attached_set)
 
         if not perform_minimization:
-            _PRINT_.info('INFO: ExtendedMinimizer::minimize("{}") no floating parameters found'
+            self.stdout.info('INFO: ExtendedMinimizer::minimize("{}") no floating parameters found'
                          '-- skipping minimization'.format(self.name))
         else:
             status = self.robust_minimize()
@@ -265,7 +266,7 @@ class ExtendedMinimizer(object):
             data_name = self.data.GetName()
             save_name = "fitresult_{}_{}".format(self.name, data_name)
             save_title = "Result of fit of p.d.f. {} to dataset {}".format(self.name, data_name)
-            _PRINT_.info('INFO: ExtendedMinimizer::minimize("{}") saving results as {}'.format(self.name, save_name))
+            self.stdout.info('INFO: ExtendedMinimizer::minimize("{}") saving results as {}'.format(self.name, save_name))
             self.fit_result = self.minimizer.save(save_name, save_title)
         # if the 'scan' option is used, minimizer will get deleted before this line
         elif self.minimizer is not None:
@@ -319,13 +320,13 @@ class ExtendedMinimizer(object):
         while ((status != 0) and (status!=1) and (strategy<2) and (retry>0)):
             strategy += 1
             retry -= 1
-            _PRINT_.error('ExtendedMinimizer::robust_minimize("{}") fit failed with status {}. '
-                  'Retrying with strategy {}'.format(self.name, status, strategy))
-            self.minimizer.setStrategy(strat)
+            self.stdout.error('ExtendedMinimizer::robust_minimize("{}") fit failed with status {}. '
+                          'Retrying with strategy {}'.format(self.name, status, strategy))
+            self.minimizer.setStrategy(strategy)
             status = self.minimizer.minimize(minimizer_type, minimizer_algo)
         
         if status not in [0, 1]:
-            _PRINT_.error('ExtendedMinimizer::robust_minimize("{}") fit failed with status {}'.format(self.name, status))
+            self.stdout.error('ExtendedMinimizer::robust_minimize("{}") fit failed with status {}'.format(self.name, status))
         
         self.minimizer.setStrategy(self.config['default_strategy'])
         
@@ -333,11 +334,11 @@ class ExtendedMinimizer(object):
     
     def use_limits(self, par:"ROOT.RooRealVar", val:float):
         if (val < par.getMin()):
-            _PRINT_.warning('ExtendedMinimizer::use_limits("{}") {} = {} limited by minimum at {}'.format(
+            self.stdout.warning('ExtendedMinimizer::use_limits("{}") {} = {} limited by minimum at {}'.format(
                             self.name, par.GetName(), val, par.getMin()))
             return par.getMin()
         elif (val > par.getMax()):
-            _PRINT_.warning('ExtendedMinimizer::use_limits("{}") {} = {} limited by maximum at {}'.format(
+            self.stdout.warning('ExtendedMinimizer::use_limits("{}") {} = {} limited by maximum at {}'.format(
                             self.name, par.GetName(), val, par.getMax()))
             return par.getMax()
         else:
@@ -374,8 +375,8 @@ class ExtendedMinimizer(object):
             _hi = hi if not math.isnan(hi) else -1.0
             v.setAsymError(_lo, _hi)
             
-            _PRINT_.info('ExtendedMinimizer::minimize("{}")'.format(self.name))        
-            _PRINT_.info(v)
+            self.stdout.info('ExtendedMinimizer::minimize("{}")'.format(self.name))        
+            self.stdout.info(v)
 
     # _____________________________________________________________________________
     # Find the value of sigma evaluated at a specified nsigma, assuming NLL -2logL is roughly parabolic in par.
@@ -411,7 +412,7 @@ class ExtendedMinimizer(object):
         temp_options = {**scan_options}
         snapshot = self.cond_set.snapshot()
         for i in range(max_iter):
-            _PRINT_.info('{} Parameter {} {}sigma '
+            self.stdout.info('{} Parameter {} {}sigma '
                          'iteration {}: start {} (MLE{})'.format(front_str, par.GetName(),
                          n_sigma, i+1, val_guess, val_guess-val_mle))
             val_pre = val_guess
@@ -444,7 +445,7 @@ class ExtendedMinimizer(object):
             for guess in guess_to_corr:
                 if (abs(guess - val_pre) < direction*val_pre*0.02):
                     damping_factor *= 0.8
-                    _PRINT_.info('{} Changing damping factor to {}'.format(front_str, damping_factor))
+                    self.stdout.info('{} Changing damping factor to {}'.format(front_str, damping_factor))
                     n_damping += 1
                     if n_damping > 10:
                         n_damping = 1
@@ -460,29 +461,29 @@ class ExtendedMinimizer(object):
             delta = val_guess - val_pre
             
             
-            _PRINT_.info('{} {} {:.3f} (MLE {:.3f}) -> {:.3f} (MLE {:.3f}), '
+            self.stdout.info('{} {} {:.3f} (MLE {:.3f}) -> {:.3f} (MLE {:.3f}), '
                           'change {:.3f}, precision {:.3f}, -2lnL {:.4f}, sigma(guess) {:.3f})'.format(
                           front_str, param_name, val_pre, val_pre - val_mle, val_guess, val_guess - val_mle,
                           delta, rel_precision, t_mu, sigma_guess))
-            _PRINT_.info('{} NLL:                 {}'.format(front_str, nll))
-            _PRINT_.info('{} delta(NLL):          {}'.format(front_str, nll - nll_min))
-            _PRINT_.info('{} nsigma*sigma(pre):   {}'.format(front_str, abs(val_pre - val_mle)))
-            _PRINT_.info('{} sigma(guess):        {}'.format(front_str, sigma_guess))
-            _PRINT_.info('{} par(guess):          {}'.format(front_str, val_guess + corr))
-            _PRINT_.info('{} best-fit val:        {}'.format(front_str, val_mle))
-            _PRINT_.info('{} tmu:                 {}'.format(front_str, t_mu))
-            _PRINT_.info('{} Precision:           {}'.format(front_str, direction*val_guess*precision))
-            _PRINT_.info('{} Correction:          {}'.format(front_str, -corr))
-            _PRINT_.info('{} nsigma*sigma(guess): {}'.format(front_str, abs(val_guess-val_mle)))
+            self.stdout.info('{} NLL:                 {}'.format(front_str, nll))
+            self.stdout.info('{} delta(NLL):          {}'.format(front_str, nll - nll_min))
+            self.stdout.info('{} nsigma*sigma(pre):   {}'.format(front_str, abs(val_pre - val_mle)))
+            self.stdout.info('{} sigma(guess):        {}'.format(front_str, sigma_guess))
+            self.stdout.info('{} par(guess):          {}'.format(front_str, val_guess + corr))
+            self.stdout.info('{} best-fit val:        {}'.format(front_str, val_mle))
+            self.stdout.info('{} tmu:                 {}'.format(front_str, t_mu))
+            self.stdout.info('{} Precision:           {}'.format(front_str, direction*val_guess*precision))
+            self.stdout.info('{} Correction:          {}'.format(front_str, -corr))
+            self.stdout.info('{} nsigma*sigma(guess): {}'.format(front_str, abs(val_guess-val_mle)))
             
             if abs(delta) <= rel_precision:
                 break
         if i >= max_iter:
-            _PRINT_.error('ERROR: find_sigma failed after {} iterations'.format(i+1))
+            self.stdout.error('ERROR: find_sigma failed after {} iterations'.format(i+1))
             return ROOT.TMath.QuietNan()
         
         err = val_guess - val_mle
-        _PRINT_.info('{} {} {}sigma = {:.3F} at -2lnL = {:4f} after {} iterations'.format(
+        self.stdout.info('{} {} {}sigma = {:.3F} at -2lnL = {:4f} after {} iterations'.format(
                      front_str, par.GetName(), n_sigma, err, t_mu, i+1))
         
         return err
