@@ -39,7 +39,7 @@ class ExtendedModel(AbstractObject):
                  data_name:Optional[str]="combData", snapshot_name:Optional[Union[List[str], str]]=None,
                  binned_likelihood:bool=True, tag_as_measurement:Optional[str]=None,
                  fix_cache:bool=True, fix_multi:bool=True, interpolation_code:int=-1,
-                 load_extension:bool=True, verbosity:Optional[Union[int, str]]=None):
+                 load_extension:bool=True, verbosity:Optional[Union[int, str]]="INFO"):
         super().__init__(verbosity=verbosity)
         self.fname = fname
         self.ws_name = ws_name
@@ -51,6 +51,7 @@ class ExtendedModel(AbstractObject):
         self.fix_cache = fix_cache
         self.fix_multi = fix_multi
         self.interpolation_code = interpolation_code
+        
         if load_extension:
             self.load_extension()
         
@@ -1041,6 +1042,8 @@ class ExtendedModel(AbstractObject):
         elif restore_states == 1:
             # load back a snapshot of the initial global observable states
             ws.loadSnapshot(nom_glob_name)
+        elif restore_states == 2:
+            pass
         else:
             raise ValueError(f"unsupported restore state option `{restore_states}`")
         
@@ -1151,19 +1154,6 @@ class ExtendedModel(AbstractObject):
             raise RuntimeError('Failed to load datasets')
         return [dataset.GetName() for dataset in datasets]
     
-    def print_summary(self, show_workspace=True, show_pois=True, show_datasets=True,
-                      show_observables=True, show_nuisance_parameters=False):
-        if show_workspace:
-            print("workspace:", self.workspace.GetName(), "\n")
-        if show_pois:
-            print("POIs:", ", ".join([i.GetName() for i in self.pois]), "\n")
-        if show_datasets:
-            print("datasets:", ", ".join([i.GetName() for i in self.workspace.allData()]), "\n")
-        if show_observables:
-            print("observables:", ", ".join([i.GetName() for i in self.observables]), "\n")
-        if show_nuisance_parameters:
-            print("nuisance parameters:", ", ".join([i.GetName() for i in self.nuisance_parameters]), "\n")
-    
     def get_category_map(self, pdf=None):
         if pdf is None:
             pdf = self.pdf
@@ -1336,4 +1326,61 @@ class ExtendedModel(AbstractObject):
                                                                    datasets, snapshots=snapshots)
         xlabel = category_map[category]['observable']
         ylabel = "Events"
-        return plot_distributions(collected_distributions[category], xlabel=xlabel, ylabel=ylabel)
\ No newline at end of file
+        return plot_distributions(collected_distributions[category], xlabel=xlabel, ylabel=ylabel)
+    
+    def print_summary(self, items:Optional[List]=None, suppress_print:bool=False, save_as:Optional[str]=None):
+        if items is None:
+            items = ['workspace', 'dataset', 'category', 'poi', 'detailed_nuisance_parameter']
+        summary_str = ""
+        # workspace
+        if 'workspace' in items:
+            summary_str += f"Workspace:\n"
+            summary_str += f"\t{self.workspace.GetName()}\n"
+        # datasets
+        if 'dataset' in items:
+            datasets = self.workspace.allData()
+            summary_str += f"Datasets:\n"
+            summary_str += "\t" + ", ".join([d.GetName() for d in datasets]) + "\n"
+        # categories
+        if 'category' in items:
+            if isinstance(self.pdf, ROOT.RooSimultaneous):
+                channel_cat = self.pdf.indexCat()
+                n_cat = len(channel_cat)
+                summary_str += f"Categories:\n"
+                for i in range(n_cat):
+                    channel_cat.setIndex(i)
+                    label = channel_cat.getLabel()
+                    summary_str += f"\t{label}\n"
+        # pois, NPs
+        param_strs     = []
+        param_sets     = []
+        if 'poi' in items:
+            pois = self.pois            
+            param_strs.append('POIs')
+            param_sets.append(pois)
+        if 'detailed_nuisance_parameter' in items:
+            constrained_nps = self.get_constrained_nuisance_parameters()
+            unconstrained_nps = self.get_unconstrained_nuisance_parameters()            
+            param_strs += ['Constrained NPs', 'Unconstrained NPs']
+            param_sets += [constrained_nps, unconstrained_nps]
+        elif 'nuisance_parameter' in items:
+            param_strs += ['NPs']
+            param_sets += [self.nuisance_parameters]
+        if 'global_observable' in items:
+            param_strs += ['Global Observables']
+            param_sets += [self.global_observables]
+        for pstr, pset in zip(param_strs, param_sets):
+            summary_str += f"{pstr}:\n"
+            for p in pset:
+                name   = p.GetName()
+                value  = p.getVal()
+                vrange = p.getRange()
+                vmin   = vrange[0]
+                vmax   = vrange[1]
+                const_str = "C" if p.isConstant() else "F"
+                summary_str += f"\t{name} = {value} [{vmin}, {vmax}] {const_str}\n"
+        if not suppress_print:
+            print(summary_str)
+        if save_as is not None:
+            with open(save_as, "w") as f:
+                f.write(summary_str)
\ No newline at end of file
